input_tables = {
    'climatology_test_code': ['climatology_test_table_A'],
    'test_any_from_asc': ['CMOR_SAMPLE_TABLE'],
    'test_any_from_asc_d': ['CMOR_SAMPLE_TABLE'],
    'test_any_from_asc_i': ['CMOR_SAMPLE_TABLE'],
    'karls_test': ['IPCC_test_table_A'],
    'test1': ['CMOR_SAMPLE_TABLE'],
    'test2': ['CMOR_SAMPLE_TABLE'],
    'test3': ['CMOR_SAMPLE_TABLE'],
    'test4': ['CMOR_SAMPLE_TABLE'],
    'main_prog': ['CMOR_SAMPLE_TABLE'],
    'test_any_from_asc_i': ['CMOR_SAMPLE_TABLE', ],
    'mytest_4d_r': ['CMOR_SAMPLE_TABLE'],
    'rewrite_harvardf_data': ['Curts_table'],
    'test_3h': ['IPCC_table_A3'],
    'test_dimensionless': ['IPCC_table_A1'],
    'test_fortran_example_00': ['IPCC_test_table_A'],
    'test_fortran_example_01': ['IPCC_test_table_A', 'IPCC_test_table_Grids'],
    'test_station_data': ['IPCC_test_table_S'],
    'test_region': ['IPCC_test_table_O'],
    'test_sigma': ['IPCC_table_A1'],
    'test_singleton': ['IPCC_test_table_A'],
    'mytest_4d_r_big_array': ['CMOR_SAMPLE_TABLE'],
    'mytest_4d_d_big_array_2': ['CMOR_SAMPLE_TABLE'],
    'mytest_4d_d_big_array_3': ['CMOR_SAMPLE_TABLE'],
    'mytest_4d_d_big_array_4': ['CMOR_SAMPLE_TABLE'],
    'mytest_4d_d_big_array_5': ['CMOR_SAMPLE_TABLE'],
    'test_lots_of_variables': ['IPCC_test_table_As'],
    'test_shrt_exp_nm_set_att_initi': ['IPCC_test_table_As'],
    'test_sophie': ['Sophie_Table'],
}
